package com.jiuqi.util;

import java.util.ArrayList;

public class SparseMatrix
{
    private ArrayList cells;
    
    public SparseMatrix() {
        this.cells = new ArrayList();
    }
    
    protected int find(final int col, final int row) {
        int sb = 0;
        int se = this.cells.size() - 1;
        int flag = -1;
        while (sb <= se) {
            final int sp = sb + se >> 1;
            final SparseCell cell = this.cells.get(sp);
            flag = cell.compareTo(col, row);
            if (flag < 0) {
                sb = sp + 1;
            }
            else {
                if (flag <= 0) {
                    sb = sp;
                    break;
                }
                se = sp - 1;
            }
        }
        return (flag == 0) ? sb : (-sb - 1);
    }
    
    public void clear() {
        this.cells.clear();
    }
    
    public int size() {
        return this.cells.size();
    }
    
    public Object get(final int index) {
        return this.cells.get(index).data;
    }
    
    public int getCol(final int index) {
        return this.cells.get(index).col;
    }
    
    public int getRow(final int index) {
        return this.cells.get(index).row;
    }
    
    public Object get(final int col, final int row) {
        final int index = this.find(col, row);
        if (index >= 0) {
            return this.cells.get(index).data;
        }
        return null;
    }
    
    public void put(final int col, final int row, final Object data) {
        int index = this.find(col, row);
        if (index >= 0) {
            if (data == null) {
                this.cells.remove(index);
            }
            else {
                this.cells.get(index).data = data;
            }
        }
        else if (data != null) {
            index = -1 - index;
            final SparseCell cell = new SparseCell();
            cell.col = col;
            cell.row = row;
            cell.data = data;
            this.cells.add(index, cell);
        }
    }
    
    public void colinsert(final int index) {
        for (int i = this.cells.size() - 1; i >= 0; --i) {
            final SparseCell cell = this.cells.get(i);
            if (cell.col >= index) {
                final SparseCell sparseCell = cell;
                ++sparseCell.col;
            }
        }
    }
    
    public void coldelete(final int index) {
        for (int i = this.cells.size() - 1; i >= 0; --i) {
            final SparseCell cell = this.cells.get(i);
            if (cell.col == index) {
                this.cells.remove(i);
            }
            else if (cell.col > index) {
                final SparseCell sparseCell = cell;
                --sparseCell.col;
            }
        }
    }
    
    public void colcountchange(final int colcount) {
        for (int i = this.cells.size() - 1; i >= 0; --i) {
            final SparseCell cell = this.cells.get(i);
            if (cell.col >= colcount) {
                this.cells.remove(i);
            }
        }
    }
    
    public void rowinsert(final int index) {
        for (int i = this.cells.size() - 1; i >= 0; --i) {
            final SparseCell cell = this.cells.get(i);
            if (cell.row >= index) {
                final SparseCell sparseCell = cell;
                ++sparseCell.row;
            }
        }
    }
    
    public void rowdelete(final int index) {
        for (int i = this.cells.size() - 1; i >= 0; --i) {
            final SparseCell cell = this.cells.get(i);
            if (cell.row == index) {
                this.cells.remove(i);
            }
            else if (cell.row > index) {
                final SparseCell sparseCell = cell;
                --sparseCell.row;
            }
        }
    }
    
    public void rowcountchange(final int rowcount) {
        for (int i = this.cells.size() - 1; i >= 0; --i) {
            final SparseCell cell = this.cells.get(i);
            if (cell.row >= rowcount) {
                this.cells.remove(i);
            }
        }
    }
}
